/************************************************************************
 *   IRC - Internet Relay Chat, src/m_list.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 * $Id: clist.c,v 1.4 2001/09/16 22:45:05 asuffield Exp $ 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "channel.h"
#include "client.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "clist.h"

#include <assert.h>
#include <string.h>
#include <stdlib.h>

int
continue_list(struct Client *cptr,
	      struct Client *sptr, int parc, char *parv[])
{
  struct Channel *chptr;
  int i, j;
  int minusers = HasUmode(sptr, UMODE_AUSPEX) ? 1 : 4;

  /* right.. if we are already involved in a "blocked" /list, we will simply
     continue where we left off */
  if (IsDoingList(sptr))
    {
      if (sptr->listprogress != -1)
	{
	  for (i = sptr->listprogress; i < CH_MAX; i++)
	    {
	      int progress2 = sptr->listprogress2;
	      for (j = 0, chptr =
		   (struct Channel *) (hash_get_channel_block(i).list);
		   (chptr) && (j < hash_get_channel_block(i).links);
		   chptr = chptr->hnextch, j++)
		{
		  if (j < progress2)
		    continue;	/* wind up to listprogress2 */
		  if (!sptr->user ||
		      (SecretChannel(chptr) && !HasUmode(sptr, UMODE_AUSPEX)
		       && !IsMember(sptr, chptr)))
		    continue;
		  if (chptr->users < minusers)
		    continue;
		  sendto_one(sptr, form_str(RPL_LIST), me.name, parv[0],
			     ShowChannel(sptr, chptr) ? chptr->chname : "*",
			     chptr->users,
			     ShowChannel(sptr, chptr) ? chptr->topic : "");
		  if (IsSendqPopped(sptr))
		    {
		      /* we popped again! : P */
		      sptr->listprogress = i;
		      sptr->listprogress2 = j;
		      return 0;
		    }
		}
	      sptr->listprogress2 = 0;
	    }
	}
      sendto_one(sptr, form_str(RPL_LISTEND), me.name, parv[0]);
      if (IsSendqPopped(sptr))
	{			/* popped with the RPL_LISTEND code. d0h */
	  sptr->listprogress = -1;
	  return 0;
	}
      ClearDoingList(sptr);	/* yupo, its over */
    }
  return 0;
}
